// file      : odb/boost/lazy-ptr.hxx
// copyright : Copyright (c) 2005-2015 Code Synthesis Tools CC
// license   : GNU GPL v2; see accompanying LICENSE file

#ifndef ODB_BOOST_LAZY_PTR_HXX
#define ODB_BOOST_LAZY_PTR_HXX

#include <odb/boost/smart-ptr/lazy-ptr.hxx>

#endif // ODB_BOOST_LAZY_PTR_HXX
